using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

//using System.Runtime.InteropServices;
using System.Collections;

using Moah; //Namespace for Firewall Controller - http://codeproject.com/KB/winsdk/WinXPSP2Firewall.aspx

//Writing to console from Windows Form: http://msdner.com/dev-archive/97/35-160-977137.shtm

//Command line : -connectTo -port -password -id -retries -autoshutdown
//host  Required
//port  Default = 5500
//password - maxlength = 7
//id - if set, means using repeater. Must be 1-999
//retries, defaults = 0
//dontautoshutdown, default = true

//Direct : -connectTo (-port 5501) (-password meh) (-retries -1) (-dontautoshutdown)
//Repeater : -connectTo (-port 5501) (-password meh) -id (-retries -1) (-dontautoshutdown)

namespace ARC2
{
    public partial class Form1 : Form
    {
        public const int WM_SYSCOMMAND = 0x112;
        public const int WM_COMMAND = 0x111;
        public const int WM_APP = 0x8000;

        private PCHWInterface pchw;

        private bool _autoShutdown = true;
        private int _reconnectTime = 0;
        private bool _connectionEstablished = false;
        private bool _reconnect = false;

        private string _targetHost;
        private string _sessionPassword;
        private string _sessionID;
        private int _targetPort;

        private delegate void DoReconnect();
        DoReconnect doReconnect;


        public Form1()
        {
            InitializeComponent();
            Initialize();

            doReconnect = new DoReconnect(this.Reconnect);
            
        }

        /// <summary>
        /// Instantiates remote control class with class name of this form, and opens firewall
        /// </summary>
        private void Initialize()
        {
            ConfigureFirewall();
            PrepareRemoteControlClass();

            textBoxViewerIP.Text = Program.targetHost;
            textBoxPort.Text = Program.targetPort.ToString();
            textBoxSessionPassword.Text = Program.sessionPassword;
            textBoxSessionID.Text = Program.sessionID;
            numericUpDown1.Value = Program.retries;

            if(!string.IsNullOrEmpty(textBoxSessionID.Text))
            {
                radioButtonViaRepeater.Checked = true;
            }

            checkBoxShutdownOnDisconnect.Checked = Program.autoShutdown;
        }

        private void ConfigureFirewall()
        {
            string appPath = Application.ExecutablePath;
            string appDescription = this.Text+" [ARC]";

            WinXPSP2FireWall fw = new WinXPSP2FireWall();
            fw.Initialize();
            fw.AddApplication(appPath, appDescription);

        }

        private void PrepareRemoteControlClass()
        {
            //Get ClassName for this Form so we can feed it to the DLL
            StringBuilder sb = new StringBuilder(256);
            NativeMethods.GetClassName(this.Handle, sb, sb.Capacity);
            string className = sb.ToString() + "\0";

            if (pchw == null)
            {
                pchw = new PCHWInterface(className);
            }
        }


        

        protected override void WndProc(ref Message m)
        {
            //Console.WriteLine((uint)m.Msg);
            

            //if (m.Msg == WM_HOTKEY)
            //{
            //    Console.Beep();
            //    // do whatever you wish to do when the hotkey is pressed
            //    // in this example we activate the form and display a messagebox
            //    this.Activate();
            //    MessageBox.Show("Hotkey has been pressed");
            //}
            
            
                if ((uint)m.Msg == WM_SYSCOMMAND)
                {
                //Console.WriteLine((uint)m.Msg);
                    uint number = (uint)m.WParam;
                    string status = "";
                    switch (number)
                    {
                        case WM_APP + 1:
                            status = "Disconnected";
                            textBoxConnectionStatus.Text = DateTime.Now + " : " + status;
                            OnDisconnect();
                            break;
                        case WM_APP + 2:
                            status = "Connected to repeater, waiting for Viewer";
                            textBoxConnectionStatus.Text = DateTime.Now + " : " + status;
                            OnConnectToRepeater();
                            break;
                        case WM_APP + 3:
                            status = "Connected to remote viewer";
                            textBoxConnectionStatus.Text = DateTime.Now + " : " + status;
                            OnConnect();
                            break;
                        case WM_APP + 4:
                            status = "Disconnecting";
                            textBoxConnectionStatus.Text = DateTime.Now + " : " + status;
                            break;
                        case WM_APP + 5:
                            status = "Exit request";
                            textBoxConnectionStatus.Text = DateTime.Now + " : " + status;
                            break;
                        default:
                            break;
                    }

                    //number -= 0x8000;

                    if (number >= WM_APP + 1 && number <= WM_APP + 5)
                    {
                        //Console.WriteLine(DateTime.Now+" Status:" + status);
                        //textBoxConnectionStatus.Text = DateTime.Now+" : "+status;
                    }
                    else
                    {
                        Console.WriteLine("Irrelevant WM_SYSCOMMAND/WM_APP message received");
                    }
                    

                    //Console.WriteLine("Number = "+number);
                    //textBoxWindowsMessages.Text = number.ToString();
                }
            
            base.WndProc(ref m);

        }

        private void OnConnect()
        {
            _connectionEstablished = true;
        }


        private void OnDisconnect()
        {
            if (_connectionEstablished || (_reconnect==false && _reconnectTime==0))
            {
                PerformAutoShutdown();
                //return;
            }
            buttonConnect.Enabled = true;
            if (_reconnect)
            {
                Console.WriteLine("219");
                doReconnect.Invoke();
                //LaunchThread();
                
            }
            else
            {
                buttonConnect.Enabled = true;
            }
            
        }

        

        private void OnConnectToRepeater()
        { 
        
        }


        private void Form1_Shown(object sender, EventArgs e)
        {
            if (Program.beginConnectionImmediately)
            {
                ValidateSettings(true);
                doReconnect.Invoke();
            }


        }

        private void radioButtonDirect_CheckedChanged(object sender, EventArgs e)
        {
            radioButtonViaRepeater.Checked = !radioButtonDirect.Checked;
            labelConnectTo.Text = "Viewer Address:";
            labelSessionID.Enabled = false;
            textBoxSessionID.Enabled = false;

        }

        private void radioButtonViaRepeater_CheckedChanged(object sender, EventArgs e)
        {
            radioButtonDirect.Checked = !radioButtonViaRepeater.Checked;
            labelConnectTo.Text = "Repeater Address:";
            labelSessionID.Enabled = true;
            textBoxSessionID.Enabled = true;
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            bool settingsOk = ValidateSettings(false);
            if (settingsOk)
            {
                Console.WriteLine("292");
                doReconnect.Invoke();
            }

        }

        /// <summary>
        /// Checks if the current settings are valid 
        /// </summary>
        /// <param name="CloseAppOnError">If true, the application is closed after displaying info</param>
        /// <returns>True if the settings are valid</returns>
        private bool ValidateSettings(bool CloseAppOnError)
        {
            bool settingsValidated = false;
            StringBuilder feedback = new StringBuilder();
            int _tmpInt;

            _targetHost = textBoxViewerIP.Text;
            if (string.IsNullOrEmpty(_targetHost))
            {
                feedback.AppendLine("Viewer/Repeater address missing");
            }

            if (string.IsNullOrEmpty(textBoxPort.Text))
            {
                textBoxPort.Text = "5500";
            }

            if (!Int32.TryParse(textBoxPort.Text, out _tmpInt))
            {
                feedback.AppendLine("Invalid port number specified");
            }
            else
            {
                _targetPort = _tmpInt;
                if (_targetPort > 65535 || _targetPort < 1)
                {
                    feedback.AppendLine("Invalid port number specified");
                }
            }

            if (textBoxSessionID.Enabled)
            {
                if (!Int32.TryParse(textBoxSessionID.Text, out _tmpInt))
                {
                    feedback.AppendLine("Session ID must be numeric");
                }
                else
                {
                    _sessionID = _tmpInt.ToString();
                    if (_tmpInt < 1 || _tmpInt > 9999)
                    {
                        feedback.AppendLine("Session ID must be between 1 and 9999");
                    }
                }
            }
            else
            {
                _sessionID = "";
            }

            _sessionPassword = textBoxSessionPassword.Text;

            if (_reconnectTime > 0)
            {
                timerIdleTimeMax.Interval = _reconnectTime * 1000 * 60;
            }

            if (_reconnectTime != 0)
            {
                _reconnect = true;
            }

            if (feedback.Length > 0)
            {
                settingsValidated = false;
                if (CloseAppOnError == true)
                {
                    timerAutoClose.Start();
                }
                
                MessageBox.Show(feedback.ToString(), "Invalid settings");

            }
            else
            {
                settingsValidated = true;

                if (_reconnectTime > 0)
                {
                    timerIdleTimeMax.Interval = _reconnectTime * 1000 * 60;
                    timerIdleTimeMax.Start();
                    Console.WriteLine("Starting timer - interval =" + timerIdleTimeMax.Interval / 60000);
                }

                if (_reconnectTime != 0)
                {
                    _reconnect = true;
                }
            }

            return settingsValidated;

        }
        private void InitTimer(int retries)
        {
            if (retries > 0)
            {
                timerIdleTimeMax.Interval = retries * 1000 * 60;
            }

            if (retries != 0)
            {
                _reconnect = true;
            }
        }

        private void Reconnect()
        {
            pchw.StopServer();
            textBoxConnectionStatus.Text = DateTime.Now + " : " + "Connecting";
            Console.WriteLine(DateTime.Now+" Status: Reconnecting");
            this.Invalidate();
            this.Refresh();
            textBoxConnectionStatus.Refresh();
            System.Threading.Thread.Sleep(2100);

            buttonConnect.Enabled = false;
            
            //PCHWCallerClass pchw = new PCHWCallerClass(className, textBoxViewerIP.Text, portNumber, tmp, textBoxSessionPassword.Text);
            pchw.Connect(_targetHost,_targetPort,_sessionID, _sessionPassword);
        }

        private void PerformAutoShutdown()
        {
            if (_autoShutdown == true)
            {
                pchw.StopServer();
                System.Threading.Thread.Sleep(4200);
                this.Close();
            }
        }

        private void buttonStopServer_Click(object sender, EventArgs e)
        {
            if (pchw == null)
                return;

            pchw.StopServer();
            timerIdleTimeMax.Stop();
            _reconnect = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void timerIdleTimeMax_Tick(object sender, EventArgs e)
        {
            timerIdleTimeMax.Stop();
            Console.WriteLine("Timeout");

            _reconnect = false;
            //Didn't get through to a viewer in time
            if (_connectionEstablished == false)
            {
                pchw.StopServer();

                PerformAutoShutdown();
            }

        }

        private void checkBoxShutdownOnDisconnect_CheckedChanged(object sender, EventArgs e)
        {
            _autoShutdown = checkBoxShutdownOnDisconnect.Checked;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            _reconnectTime = (int)numericUpDown1.Value;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
         
        }
        private void timerAutoClose_Tick(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonCmdLineArgsInfo_Click(object sender, EventArgs e)
        {
            string msg = "Direct : -connectTo hostname (-port 5500) (-password pwd) (-retries -1) (-dontautoshutdown)\n" +
            "Repeater : -connectTo hostname (-port 5500) (-password pwd) -id asdf (-retries -1) (-dontautoshutdown)\n" +
            "Passing ccommand line arguments will override settings in .exe.config";

            MessageBox.Show(msg,"Use of Command Line Arguments");
        }

        private void radioButtonViaRepeater_Click(object sender, EventArgs e)
        {
            ActiveControl = textBoxSessionID;
        }

        private void radioButtonDirect_Click(object sender, EventArgs e)
        {
            ActiveControl = textBoxViewerIP;
        }


        




    }


}